'use strict';

var shim = require('use-sync-external-store/shim');

var __DEV__ = process.env.NODE_ENV !== 'production';
var isInFunction = false;
var run = function (fn) {
    fn();
};
var resso = function (store) {
    if (__DEV__ && Object.prototype.toString.call(store) !== '[object Object]') {
        throw new Error('object required');
    }
    var state = {};
    Object.keys(store).forEach(function (key) {
        if (typeof store[key] === 'function') {
            return;
        }
        var listeners = new Set();
        state[key] = {
            subscribe: function (listener) {
                listeners.add(listener);
                return function () { return listeners.delete(listener); };
            },
            getSnapshot: function () { return store[key]; },
            setSnapshot: function (val) {
                if (val !== store[key]) {
                    store[key] = val;
                    run(function () { return listeners.forEach(function (listener) { return listener(); }); });
                }
            },
            useSnapshot: function () {
                return shim.useSyncExternalStore(state[key].subscribe, state[key].getSnapshot);
            },
        };
    });
    return new Proxy(store, {
        get: function (_, key) {
            if (typeof store[key] === 'function') {
                return function () {
                    var args = [];
                    for (var _i = 0; _i < arguments.length; _i++) {
                        args[_i] = arguments[_i];
                    }
                    isInFunction = true;
                    var res = store[key].apply(store, args);
                    isInFunction = false;
                    return res;
                };
            }
            if (isInFunction) {
                return store[key];
            }
            try {
                return state[key].useSnapshot();
            }
            catch (err) {
                return store[key];
            }
        },
        set: function (_, key, val) {
            if (typeof store[key] === 'function') {
                return true;
            }
            state[key].setSnapshot(val);
            return true;
        },
    });
};
resso.config = function (_a) {
    var batch = _a.batch;
    run = batch;
};

module.exports = resso;
